// USBio.h: interface for the sdr14USBio class.
//  A class for communicating with the SDR-14 over a USB port
//////////////////////////////////////////////////////////////////////
// Copyright 2003.    Moe Wheatley AE4JY  <http://www.moetronix.com>
//
//This program is free software; you can redistribute it and/or
//modify it how ever you wish.  Its primary goal is to provide an
// example for 3rd party software developers.  Support will be limited
// depending on my available time.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
////////////////////////////////////////////////////////////////////////

#if !defined(SDR14_USB_IO)
#define SDR14_USB_IO

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#pragma pack(push)
#pragma pack(1)

#include "SDR14FTD2XX.h"
#include "SDR14params.h"
#include "SDR14ascpDefs.h"	//local copy of ASCP defs

#define EEROM_SIZE 32

typedef FT14_STATUS (WINAPI *PtrFTOpen)(PVOID, FT14_HANDLE *); 
typedef FT14_STATUS (WINAPI *PtrFTOpenEx)(PVOID, DWORD, FT14_HANDLE *); 
typedef FT14_STATUS (WINAPI *PtrFTListDevices)(PVOID, PVOID, DWORD);
typedef FT14_STATUS (WINAPI *PtrFTGetStatus)(FT14_HANDLE,LPDWORD,LPDWORD,LPDWORD);
typedef FT14_STATUS (WINAPI *PtrFTClose)(FT14_HANDLE);
typedef FT14_STATUS (WINAPI *PtrFTRead)(FT14_HANDLE, LPVOID, DWORD, LPDWORD);
typedef FT14_STATUS (WINAPI *PtrFTWrite)(FT14_HANDLE, LPVOID, DWORD, LPDWORD);
typedef FT14_STATUS (WINAPI *PtrFTResetDevice)(FT14_HANDLE);
typedef FT14_STATUS (WINAPI *PtrFTPurge)(FT14_HANDLE, ULONG);
typedef FT14_STATUS (WINAPI *PtrFTSetTimeouts)(FT14_HANDLE, ULONG, ULONG);
typedef FT14_STATUS (WINAPI *PtrFTGetQueueStatus)(FT14_HANDLE, LPDWORD);
typedef FT14_STATUS (WINAPI *PtrFTSetEventNotification)(FT14_HANDLE, DWORD,PVOID);
typedef FT14_STATUS (WINAPI *PtrFTSetLatencyTimer)(FT14_HANDLE, UCHAR);
typedef FT14_STATUS (WINAPI *PtrFT14_EE_Read)(FT14_HANDLE, PFT14_PROGRAM_DATA);
typedef FT14_STATUS (WINAPI *PtrFT14_SetResetPipeRetryCount)(FT14_HANDLE, DWORD);
typedef FT14_STATUS (WINAPI *PtrFT14_SetUSBParameters)( FT14_HANDLE ftHandle, ULONG ulInTransferSize, ULONG ulOutTransferSize);
typedef FT14_STATUS (WINAPI *PtrFT14_EE_UARead)( FT14_HANDLE ftHandle, PUCHAR pucData, DWORD dwDataLen,LPDWORD lpdwBytesRead);
typedef FT14_STATUS (WINAPI *PtrFT14_EE_UAWrite)( FT14_HANDLE ftHandle, PUCHAR pucData, DWORD dwDataLen);


//////////////////////////////////////////////////////////////////////////////
// highly non-portable ASCP message buffer structure but....its fast and easy
//////////////////////////////////////////////////////////////////////////////
typedef union
{
	struct bts
	{
		unsigned __int8 b0;
		unsigned __int8 b1;
		unsigned __int8 b2;
		unsigned __int8 b3;
		unsigned __int8 parms[MAX_MSG_LENGTH-4];
	}bytes;
	struct dbts
	{
		unsigned __int8 b0;
		unsigned __int8 b1;
		unsigned __int8 parms[MAX_MSG_LENGTH-2];
	}dibytes;
	struct dwrds
	{
		unsigned __int16 header;
		unsigned __int16 parms[(MAX_MSG_LENGTH-2)/2];
	}diwords;
	struct wrds
	{
		unsigned __int16 header;
		unsigned __int16 CItem;
		unsigned __int16 parms[(MAX_MSG_LENGTH-4)/2];
	}words;
	struct dlngs
	{
		unsigned __int16 header;
		unsigned __int16 parms[(MAX_MSG_LENGTH-2)/2];
	}data;
	struct lngs
	{
		unsigned __int32 header;
		unsigned __int32 parms[(MAX_MSG_LENGTH-4)/4];
	}longs;
	unsigned __int8 buf[MAX_MSG_LENGTH];
}ASCPstruct;

class sdr14USBio  
{
public:

	HWND	m_MSGhWnd;
	PtrFTOpen						g_pOpen; 
	PtrFTOpenEx						g_pOpenEx; 
	PtrFTListDevices				g_pListDevices; 
	PtrFTGetStatus					g_pGetStatus; 
	PtrFTClose						g_pClose;
	PtrFTRead						g_pRead;
	PtrFTWrite						g_pWrite;
	PtrFTResetDevice				g_pResetDevice;
	PtrFTPurge						g_pPurge;
	PtrFTSetTimeouts				g_pSetTimeouts;
	PtrFTGetQueueStatus				g_pGetQueueStatus;
	PtrFTSetEventNotification		g_pSetEventNotification;
	PtrFTSetLatencyTimer			g_pSetLatencyTimer;
	PtrFT14_EE_Read					g_pFT14_EE_Read;
	PtrFT14_SetResetPipeRetryCount	g_pFT14_SetResetPipeRetryCount;
	PtrFT14_SetUSBParameters		g_pFT14_SetUSBParameters;
	PtrFT14_EE_UARead				g_pFT14_EE_UARead;
	PtrFT14_EE_UAWrite				g_pFT14_EE_UAWrite;

	BOOL m_USBPortOpen;
	BOOL IsPortOpen(){ return m_USBPortOpen;};
	sdr14USBio();
	virtual ~sdr14USBio();
	// Externally exposed functions
	int Load6620Params( int filter );
	void USBthreadProc(  LPVOID pCtrl );
	void FIFOThreadProc(LPVOID pCtrl);
	void SendASCPMsg( ASCPstruct* pASCPmsg);
	void ReadIOMemory( int adr, unsigned int* pData);
	void WriteIOMemory( int adr, unsigned int Data);
	// Externally exposed variables
	BOOL m_Running;
	int m_Last6620FilterLoaded;

	// local functions
	void Send6620Msg( int adr, unsigned int data);
	void CloseUSBPort();
	void DeleteResources();
	BOOL OpenUSBdll();
	void SetSDRIQSampleRateFromEEROM(unsigned int rate);

	// local variables
	int m_RxMsgLength;
	int m_RxMsgIndex;
	int m_MsgState;
	int m_FifoHead;
	int m_FifoTail;
	unsigned char m_EEROMData[EEROM_SIZE];
	unsigned __int8* m_pUSBBuffer;
	unsigned __int8* m_pUSBFifo;
	FT14_STATUS m_FTStatus;
	FT14_HANDLE m_FTHandle;
	HMODULE h_FTlib;
	HANDLE m_hReadEvent;
	HANDLE m_hFifoWriteEvent;
	FT14_PROGRAM_DATA m_FT14_ProgramData;
	CRITICAL_SECTION m_CriticalSection;	//for keeping threads from stomping on each other
	ASCPstruct m_rxASCPmsg;
	ASCPstruct m_txASCPmsg;
	C6620Params m_6620params;
};

#pragma pack(pop)
#endif // !defined(SDR14_USB_IO)
